package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public final class TwofishEngine implements BlockCipher {
   private static final byte[][] P;
   private static final int P_00 = 1;
   private static final int P_01 = 0;
   private static final int P_02 = 0;
   private static final int P_03 = 1;
   private static final int P_04 = 1;
   private static final int P_10 = 0;
   private static final int P_11 = 0;
   private static final int P_12 = 1;
   private static final int P_13 = 1;
   private static final int P_14 = 0;
   private static final int P_20 = 1;
   private static final int P_21 = 1;
   private static final int P_22 = 0;
   private static final int P_23 = 0;
   private static final int P_24 = 0;
   private static final int P_30 = 0;
   private static final int P_31 = 1;
   private static final int P_32 = 1;
   private static final int P_33 = 0;
   private static final int P_34 = 1;
   private static final int GF256_FDBK = 361;
   private static final int GF256_FDBK_2 = 180;
   private static final int GF256_FDBK_4 = 90;
   private static final int RS_GF_FDBK = 333;
   private static final int ROUNDS = 16;
   private static final int MAX_ROUNDS = 16;
   private static final int BLOCK_SIZE = 16;
   private static final int MAX_KEY_BITS = 256;
   private static final int INPUT_WHITEN = 0;
   private static final int OUTPUT_WHITEN = 4;
   private static final int ROUND_SUBKEYS = 8;
   private static final int TOTAL_SUBKEYS = 40;
   private static final int SK_STEP = 33686018;
   private static final int SK_BUMP = 16843009;
   private static final int SK_ROTL = 9;
   private boolean encrypting = false;
   private int[] gMDS0 = new int[256];
   private int[] gMDS1 = new int[256];
   private int[] gMDS2 = new int[256];
   private int[] gMDS3 = new int[256];
   private int[] gSubKeys;
   private int[] gSBox;
   private int k64Cnt = 0;
   private byte[] workingKey = null;

   static {
      byte[][] var10000 = new byte[2][];
      byte[] var0;
      byte[] var10003 = var0 = new byte[256];
      var10003[0] = -87;
      var10003[1] = 103;
      var10003[2] = -77;
      var10003[3] = -24;
      var10003[4] = 4;
      var10003[5] = -3;
      var10003[6] = -93;
      var10003[7] = 118;
      var10003[8] = -102;
      var10003[9] = -110;
      var10003[10] = -128;
      var10003[11] = 120;
      var10003[12] = -28;
      var10003[13] = -35;
      var10003[14] = -47;
      var10003[15] = 56;
      var10003[16] = 13;
      var10003[17] = -58;
      var10003[18] = 53;
      var10003[19] = -104;
      var10003[20] = 24;
      var10003[21] = -9;
      var10003[22] = -20;
      var10003[23] = 108;
      var10003[24] = 67;
      var10003[25] = 117;
      var10003[26] = 55;
      var10003[27] = 38;
      var10003[28] = -6;
      var10003[29] = 19;
      var10003[30] = -108;
      var10003[31] = 72;
      var10003[32] = -14;
      var10003[33] = -48;
      var10003[34] = -117;
      var10003[35] = 48;
      var10003[36] = -124;
      var10003[37] = 84;
      var10003[38] = -33;
      var10003[39] = 35;
      var10003[40] = 25;
      var10003[41] = 91;
      var10003[42] = 61;
      var10003[43] = 89;
      var10003[44] = -13;
      var10003[45] = -82;
      var10003[46] = -94;
      var10003[47] = -126;
      var10003[48] = 99;
      var10003[49] = 1;
      var10003[50] = -125;
      var10003[51] = 46;
      var10003[52] = -39;
      var10003[53] = 81;
      var10003[54] = -101;
      var10003[55] = 124;
      var10003[56] = -90;
      var10003[57] = -21;
      var10003[58] = -91;
      var10003[59] = -66;
      var10003[60] = 22;
      var10003[61] = 12;
      var10003[62] = -29;
      var10003[63] = 97;
      var10003[64] = -64;
      var10003[65] = -116;
      var10003[66] = 58;
      var10003[67] = -11;
      var10003[68] = 115;
      var10003[69] = 44;
      var10003[70] = 37;
      var10003[71] = 11;
      var10003[72] = -69;
      var10003[73] = 78;
      var10003[74] = -119;
      var10003[75] = 107;
      var10003[76] = 83;
      var10003[77] = 106;
      var10003[78] = -76;
      var10003[79] = -15;
      var10003[80] = -31;
      var10003[81] = -26;
      var10003[82] = -67;
      var10003[83] = 69;
      var10003[84] = -30;
      var10003[85] = -12;
      var10003[86] = -74;
      var10003[87] = 102;
      var10003[88] = -52;
      var10003[89] = -107;
      var10003[90] = 3;
      var10003[91] = 86;
      var10003[92] = -44;
      var10003[93] = 28;
      var10003[94] = 30;
      var10003[95] = -41;
      var10003[96] = -5;
      var10003[97] = -61;
      var10003[98] = -114;
      var10003[99] = -75;
      var10003[100] = -23;
      var10003[101] = -49;
      var10003[102] = -65;
      var10003[103] = -70;
      var10003[104] = -22;
      var10003[105] = 119;
      var10003[106] = 57;
      var10003[107] = -81;
      var10003[108] = 51;
      var10003[109] = -55;
      var10003[110] = 98;
      var10003[111] = 113;
      var10003[112] = -127;
      var10003[113] = 121;
      var10003[114] = 9;
      var10003[115] = -83;
      var10003[116] = 36;
      var10003[117] = -51;
      var10003[118] = -7;
      var10003[119] = -40;
      var10003[120] = -27;
      var10003[121] = -59;
      var10003[122] = -71;
      var10003[123] = 77;
      var10003[124] = 68;
      var10003[125] = 8;
      var10003[126] = -122;
      var10003[127] = -25;
      var10003[128] = -95;
      var10003[129] = 29;
      var10003[130] = -86;
      var10003[131] = -19;
      var10003[132] = 6;
      var10003[133] = 112;
      var10003[134] = -78;
      var10003[135] = -46;
      var10003[136] = 65;
      var10003[137] = 123;
      var10003[138] = -96;
      var10003[139] = 17;
      var10003[140] = 49;
      var10003[141] = -62;
      var10003[142] = 39;
      var10003[143] = -112;
      var10003[144] = 32;
      var10003[145] = -10;
      var10003[146] = 96;
      var10003[147] = -1;
      var10003[148] = -106;
      var10003[149] = 92;
      var10003[150] = -79;
      var10003[151] = -85;
      var10003[152] = -98;
      var10003[153] = -100;
      var10003[154] = 82;
      var10003[155] = 27;
      var10003[156] = 95;
      var10003[157] = -109;
      var10003[158] = 10;
      var10003[159] = -17;
      var10003[160] = -111;
      var10003[161] = -123;
      var10003[162] = 73;
      var10003[163] = -18;
      var10003[164] = 45;
      var10003[165] = 79;
      var10003[166] = -113;
      var10003[167] = 59;
      var10003[168] = 71;
      var10003[169] = -121;
      var10003[170] = 109;
      var10003[171] = 70;
      var10003[172] = -42;
      var10003[173] = 62;
      var10003[174] = 105;
      var10003[175] = 100;
      var10003[176] = 42;
      var10003[177] = -50;
      var10003[178] = -53;
      var10003[179] = 47;
      var10003[180] = -4;
      var10003[181] = -105;
      var10003[182] = 5;
      var10003[183] = 122;
      var10003[184] = -84;
      var10003[185] = 127;
      var10003[186] = -43;
      var10003[187] = 26;
      var10003[188] = 75;
      var10003[189] = 14;
      var10003[190] = -89;
      var10003[191] = 90;
      var10003[192] = 40;
      var10003[193] = 20;
      var10003[194] = 63;
      var10003[195] = 41;
      var10003[196] = -120;
      var10003[197] = 60;
      var10003[198] = 76;
      var10003[199] = 2;
      var10003[200] = -72;
      var10003[201] = -38;
      var10003[202] = -80;
      var10003[203] = 23;
      var10003[204] = 85;
      var10003[205] = 31;
      var10003[206] = -118;
      var10003[207] = 125;
      var10003[208] = 87;
      var10003[209] = -57;
      var10003[210] = -115;
      var10003[211] = 116;
      var10003[212] = -73;
      var10003[213] = -60;
      var10003[214] = -97;
      var10003[215] = 114;
      var10003[216] = 126;
      var10003[217] = 21;
      var10003[218] = 34;
      var10003[219] = 18;
      var10003[220] = 88;
      var10003[221] = 7;
      var10003[222] = -103;
      var10003[223] = 52;
      var10003[224] = 110;
      var10003[225] = 80;
      var10003[226] = -34;
      var10003[227] = 104;
      var10003[228] = 101;
      var10003[229] = -68;
      var10003[230] = -37;
      var10003[231] = -8;
      var10003[232] = -56;
      var10003[233] = -88;
      var10003[234] = 43;
      var10003[235] = 64;
      var10003[236] = -36;
      var10003[237] = -2;
      var10003[238] = 50;
      var10003[239] = -92;
      var10003[240] = -54;
      var10003[241] = 16;
      var10003[242] = 33;
      var10003[243] = -16;
      var10003[244] = -45;
      var10003[245] = 93;
      var10003[246] = 15;
      var10003[248] = 111;
      var10003[249] = -99;
      var10003[250] = 54;
      var10003[251] = 66;
      var10003[252] = 74;
      var10003[253] = 94;
      var10003[254] = -63;
      var10003[255] = -32;
      var10000[0] = var0;
      byte[] var10002 = var0 = new byte[256];
      var10002[0] = 117;
      var10002[1] = -13;
      var10002[2] = -58;
      var10002[3] = -12;
      var10002[4] = -37;
      var10002[5] = 123;
      var10002[6] = -5;
      var10002[7] = -56;
      var10002[8] = 74;
      var10002[9] = -45;
      var10002[10] = -26;
      var10002[11] = 107;
      var10002[12] = 69;
      var10002[13] = 125;
      var10002[14] = -24;
      var10002[15] = 75;
      var10002[16] = -42;
      var10002[17] = 50;
      var10002[18] = -40;
      var10002[19] = -3;
      var10002[20] = 55;
      var10002[21] = 113;
      var10002[22] = -15;
      var10002[23] = -31;
      var10002[24] = 48;
      var10002[25] = 15;
      var10002[26] = -8;
      var10002[27] = 27;
      var10002[28] = -121;
      var10002[29] = -6;
      var10002[30] = 6;
      var10002[31] = 63;
      var10002[32] = 94;
      var10002[33] = -70;
      var10002[34] = -82;
      var10002[35] = 91;
      var10002[36] = -118;
      var10002[38] = -68;
      var10002[39] = -99;
      var10002[40] = 109;
      var10002[41] = -63;
      var10002[42] = -79;
      var10002[43] = 14;
      var10002[44] = -128;
      var10002[45] = 93;
      var10002[46] = -46;
      var10002[47] = -43;
      var10002[48] = -96;
      var10002[49] = -124;
      var10002[50] = 7;
      var10002[51] = 20;
      var10002[52] = -75;
      var10002[53] = -112;
      var10002[54] = 44;
      var10002[55] = -93;
      var10002[56] = -78;
      var10002[57] = 115;
      var10002[58] = 76;
      var10002[59] = 84;
      var10002[60] = -110;
      var10002[61] = 116;
      var10002[62] = 54;
      var10002[63] = 81;
      var10002[64] = 56;
      var10002[65] = -80;
      var10002[66] = -67;
      var10002[67] = 90;
      var10002[68] = -4;
      var10002[69] = 96;
      var10002[70] = 98;
      var10002[71] = -106;
      var10002[72] = 108;
      var10002[73] = 66;
      var10002[74] = -9;
      var10002[75] = 16;
      var10002[76] = 124;
      var10002[77] = 40;
      var10002[78] = 39;
      var10002[79] = -116;
      var10002[80] = 19;
      var10002[81] = -107;
      var10002[82] = -100;
      var10002[83] = -57;
      var10002[84] = 36;
      var10002[85] = 70;
      var10002[86] = 59;
      var10002[87] = 112;
      var10002[88] = -54;
      var10002[89] = -29;
      var10002[90] = -123;
      var10002[91] = -53;
      var10002[92] = 17;
      var10002[93] = -48;
      var10002[94] = -109;
      var10002[95] = -72;
      var10002[96] = -90;
      var10002[97] = -125;
      var10002[98] = 32;
      var10002[99] = -1;
      var10002[100] = -97;
      var10002[101] = 119;
      var10002[102] = -61;
      var10002[103] = -52;
      var10002[104] = 3;
      var10002[105] = 111;
      var10002[106] = 8;
      var10002[107] = -65;
      var10002[108] = 64;
      var10002[109] = -25;
      var10002[110] = 43;
      var10002[111] = -30;
      var10002[112] = 121;
      var10002[113] = 12;
      var10002[114] = -86;
      var10002[115] = -126;
      var10002[116] = 65;
      var10002[117] = 58;
      var10002[118] = -22;
      var10002[119] = -71;
      var10002[120] = -28;
      var10002[121] = -102;
      var10002[122] = -92;
      var10002[123] = -105;
      var10002[124] = 126;
      var10002[125] = -38;
      var10002[126] = 122;
      var10002[127] = 23;
      var10002[128] = 102;
      var10002[129] = -108;
      var10002[130] = -95;
      var10002[131] = 29;
      var10002[132] = 61;
      var10002[133] = -16;
      var10002[134] = -34;
      var10002[135] = -77;
      var10002[136] = 11;
      var10002[137] = 114;
      var10002[138] = -89;
      var10002[139] = 28;
      var10002[140] = -17;
      var10002[141] = -47;
      var10002[142] = 83;
      var10002[143] = 62;
      var10002[144] = -113;
      var10002[145] = 51;
      var10002[146] = 38;
      var10002[147] = 95;
      var10002[148] = -20;
      var10002[149] = 118;
      var10002[150] = 42;
      var10002[151] = 73;
      var10002[152] = -127;
      var10002[153] = -120;
      var10002[154] = -18;
      var10002[155] = 33;
      var10002[156] = -60;
      var10002[157] = 26;
      var10002[158] = -21;
      var10002[159] = -39;
      var10002[160] = -59;
      var10002[161] = 57;
      var10002[162] = -103;
      var10002[163] = -51;
      var10002[164] = -83;
      var10002[165] = 49;
      var10002[166] = -117;
      var10002[167] = 1;
      var10002[168] = 24;
      var10002[169] = 35;
      var10002[170] = -35;
      var10002[171] = 31;
      var10002[172] = 78;
      var10002[173] = 45;
      var10002[174] = -7;
      var10002[175] = 72;
      var10002[176] = 79;
      var10002[177] = -14;
      var10002[178] = 101;
      var10002[179] = -114;
      var10002[180] = 120;
      var10002[181] = 92;
      var10002[182] = 88;
      var10002[183] = 25;
      var10002[184] = -115;
      var10002[185] = -27;
      var10002[186] = -104;
      var10002[187] = 87;
      var10002[188] = 103;
      var10002[189] = 127;
      var10002[190] = 5;
      var10002[191] = 100;
      var10002[192] = -81;
      var10002[193] = 99;
      var10002[194] = -74;
      var10002[195] = -2;
      var10002[196] = -11;
      var10002[197] = -73;
      var10002[198] = 60;
      var10002[199] = -91;
      var10002[200] = -50;
      var10002[201] = -23;
      var10002[202] = 104;
      var10002[203] = 68;
      var10002[204] = -32;
      var10002[205] = 77;
      var10002[206] = 67;
      var10002[207] = 105;
      var10002[208] = 41;
      var10002[209] = 46;
      var10002[210] = -84;
      var10002[211] = 21;
      var10002[212] = 89;
      var10002[213] = -88;
      var10002[214] = 10;
      var10002[215] = -98;
      var10002[216] = 110;
      var10002[217] = 71;
      var10002[218] = -33;
      var10002[219] = 52;
      var10002[220] = 53;
      var10002[221] = 106;
      var10002[222] = -49;
      var10002[223] = -36;
      var10002[224] = 34;
      var10002[225] = -55;
      var10002[226] = -64;
      var10002[227] = -101;
      var10002[228] = -119;
      var10002[229] = -44;
      var10002[230] = -19;
      var10002[231] = -85;
      var10002[232] = 18;
      var10002[233] = -94;
      var10002[234] = 13;
      var10002[235] = 82;
      var10002[236] = -69;
      var10002[237] = 2;
      var10002[238] = 47;
      var10002[239] = -87;
      var10002[240] = -41;
      var10002[241] = 97;
      var10002[242] = 30;
      var10002[243] = -76;
      var10002[244] = 80;
      var10002[245] = 4;
      var10002[246] = -10;
      var10002[247] = -62;
      var10002[248] = 22;
      var10002[249] = 37;
      var10002[250] = -122;
      var10002[251] = 86;
      var10002[252] = 85;
      var10002[253] = 9;
      var10002[254] = -66;
      var10002[255] = -111;
      var10000[1] = var0;
      P = var10000;
   }

   public TwofishEngine() {
      int[] var1 = new int[2];
      int[] var2 = new int[2];
      int[] var3 = new int[2];

      for(int var4 = 0; var4 < 256; ++var4) {
         byte[][] var5;
         byte[][] var10009 = var5 = P;
         int var6 = var5[0][var4] & 255;
         var1[0] = var6;
         var2[0] = this.Mx_X(var6) & 255;
         var3[0] = this.Mx_Y(var6) & 255;
         var6 = var10009[1][var4] & 255;
         var1[1] = var6;
         var2[1] = this.Mx_X(var6) & 255;
         var3[1] = this.Mx_Y(var6) & 255;
         this.gMDS0[var4] = var1[1] | var2[1] << 8 | var3[1] << 16 | var3[1] << 24;
         this.gMDS1[var4] = var3[0] | var3[0] << 8 | var2[0] << 16 | var1[0] << 24;
         this.gMDS2[var4] = var2[1] | var3[1] << 8 | var1[1] << 16 | var3[1] << 24;
         this.gMDS3[var4] = var2[0] | var1[0] << 8 | var3[0] << 16 | var2[0] << 24;
      }

   }

   private void setKey(byte[] key) {
      int[] var2 = new int[4];
      int[] var3 = new int[4];
      int[] var4 = new int[4];
      this.gSubKeys = new int[40];
      int var5;
      if ((var5 = this.k64Cnt) < 1) {
         throw new IllegalArgumentException("Key size less than 64 bits");
      } else if (var5 > 4) {
         throw new IllegalArgumentException("Key size larger than 256 bits");
      } else {
         int var6;
         int var7;
         for(var5 = 0; var5 < this.k64Cnt; ++var5) {
            var2[var5] = this.BytesTo32Bits(key, var6 = var5 * 8);
            var3[var5] = this.BytesTo32Bits(key, var6 + 4);
            var6 = this.k64Cnt - 1 - var5;
            var7 = var2[var5];
            var4[var6] = this.RS_MDS_Encode(var7, var3[var5]);
         }

         int key1;
         for(key1 = 0; key1 < 20; ++key1) {
            var6 = this.F32(var5 = key1 * 33686018, var2) + (var5 = (var5 = this.F32(var5 + 16843009, var3)) << 8 | var5 >>> 24);
            int[] var17;
            int[] var10000 = var17 = this.gSubKeys;
            int var10002 = var5;
            var17[var5 = key1 * 2] = var6;
            int var10001 = var6 += var10002;
            ++var5;
            var10000[var5] = var10001 << 9 | var6 >>> 23;
         }

         var5 = var4[0];
         var6 = var4[1];
         var7 = var4[2];
         int var8 = var4[3];
         this.gSBox = new int[1024];

         for(key1 = 0; key1 < 256; ++key1) {
            int var9;
            int var13;
            int var14;
            int var16;
            label37: {
               byte[][] var18;
               switch (this.k64Cnt & 3) {
                  case 0:
                     var18 = P;
                     var13 = var18[1][key1] & 255 ^ this.b0(var8);
                     var14 = var18[0][key1] & 255 ^ this.b1(var8);
                     var16 = var18[0][key1] & 255 ^ this.b2(var8);
                     var9 = var18[1][key1] & 255 ^ this.b3(var8);
                     break;
                  case 1:
                     byte[][] var15;
                     this.gSBox[var13 = key1 * 2] = this.gMDS0[(var15 = P)[0][key1] & 255 ^ this.b0(var5)];
                     this.gSBox[var13 + 1] = this.gMDS1[var15[0][key1] & 255 ^ this.b1(var5)];
                     this.gSBox[var13 + 512] = this.gMDS2[var15[1][key1] & 255 ^ this.b2(var5)];
                     this.gSBox[var13 + 513] = this.gMDS3[var15[1][key1] & 255 ^ this.b3(var5)];
                     continue;
                  case 2:
                     var9 = key1;
                     var16 = key1;
                     var14 = key1;
                     var13 = key1;
                     break label37;
                  case 3:
                     var9 = key1;
                     var16 = key1;
                     var14 = key1;
                     var13 = key1;
                     break;
                  default:
                     continue;
               }

               var18 = P;
               var13 = var18[1][var13] & 255 ^ this.b0(var7);
               var14 = var18[1][var14] & 255 ^ this.b1(var7);
               var16 = var18[0][var16] & 255 ^ this.b2(var7);
               var9 = var18[0][var9] & 255 ^ this.b3(var7);
            }

            int var10;
            byte[][] var11;
            this.gSBox[var10 = key1 * 2] = this.gMDS0[(var11 = P)[0][var11[0][var13] & 255 ^ this.b0(var6)] & 255 ^ this.b0(var5)];
            this.gSBox[var10 + 1] = this.gMDS1[var11[0][var11[1][var14] & 255 ^ this.b1(var6)] & 255 ^ this.b1(var5)];
            this.gSBox[var10 + 512] = this.gMDS2[var11[1][var11[0][var16] & 255 ^ this.b2(var6)] & 255 ^ this.b2(var5)];
            this.gSBox[var10 + 513] = this.gMDS3[var11[1][var11[1][var9] & 255 ^ this.b3(var6)] & 255 ^ this.b3(var5)];
         }

      }
   }

   private void encryptBlock(byte[] src, int srcIndex, byte[] dst, int dstIndex) {
      int var10002 = srcIndex;
      int var10005 = srcIndex;
      int src1 = this.BytesTo32Bits(src, srcIndex) ^ this.gSubKeys[0];
      srcIndex = this.BytesTo32Bits(src, srcIndex + 4) ^ this.gSubKeys[1];
      int var5 = this.BytesTo32Bits(src, var10005 + 8) ^ this.gSubKeys[2];
      int var6 = this.BytesTo32Bits(src, var10002 + 12) ^ this.gSubKeys[3];
      int var7 = 8;

      int var11;
      for(int var8 = 0; var8 < 16; src1 = var11) {
         int var13 = src1;
         int[] var9;
         int var10 = (var10 = var5 ^ (src1 = this.Fe32_0(src1)) + (var5 = this.Fe32_3(srcIndex)) + (var9 = this.gSubKeys)[var7++]) >>> 1 | var10 << 31;
         int var10003 = var6 << 1 | var6 >>> 31;
         int var15 = src1 + var5 * 2;
         src1 = var7 + 1;
         var6 = var10003 ^ var15 + var9[var7];
         var11 = (var11 = var13 ^ (var5 = this.Fe32_0(var10)) + (var7 = this.Fe32_3(var6)) + (var9 = this.gSubKeys)[src1++]) >>> 1 | var11 << 31;
         int var10000 = srcIndex << 1 | srcIndex >>> 31;
         var13 = var5 + var7 * 2;
         var7 = src1 + 1;
         srcIndex = var10000 ^ var13 + var9[src1];
         var8 += 2;
         var5 = var10;
      }

      this.Bits32ToBytes(var5 ^ this.gSubKeys[4], dst, dstIndex);
      this.Bits32ToBytes(var6 ^ this.gSubKeys[5], dst, dstIndex + 4);
      this.Bits32ToBytes(src1 ^ this.gSubKeys[6], dst, dstIndex + 8);
      this.Bits32ToBytes(srcIndex ^ this.gSubKeys[7], dst, dstIndex + 12);
   }

   private void decryptBlock(byte[] src, int srcIndex, byte[] dst, int dstIndex) {
      byte[] var10001 = src;
      int var10002 = srcIndex;
      byte[] var10004 = src;
      int var10005 = srcIndex;
      byte[] var10007 = src;
      int src1 = this.BytesTo32Bits(src, srcIndex) ^ this.gSubKeys[4];
      srcIndex = this.BytesTo32Bits(var10007, srcIndex + 4) ^ this.gSubKeys[5];
      int var5 = this.BytesTo32Bits(var10004, var10005 + 8) ^ this.gSubKeys[6];
      int var6 = this.BytesTo32Bits(var10001, var10002 + 12) ^ this.gSubKeys[7];
      int var7 = 39;

      for(int var8 = 0; var8 < 16; var8 += 2) {
         int var10000 = srcIndex;
         var10002 = var6;
         int var10003 = (var6 = this.Fe32_0(src1)) + (srcIndex = this.Fe32_3(srcIndex)) * 2;
         int[] var9;
         int[] var17 = var9 = this.gSubKeys;
         var10005 = var7;
         var7 += -1;
         int var10;
         var10002 = var10 = var10002 ^ var10003 + var17[var10005];
         var10003 = var5 << 1 | var5 >>> 31;
         int var18 = var6 + srcIndex;
         srcIndex = var7 + -1;
         var5 = var10003 ^ var18 + var9[var7];
         var6 = var10002 >>> 1 | var10 << 31;
         int var13;
         int var15 = (var7 = this.Fe32_0(var5)) + (var13 = this.Fe32_3(var6)) * 2;
         int[] var14;
         int[] var16 = var14 = this.gSubKeys;
         var10003 = srcIndex;
         srcIndex += -1;
         int var11;
         var10000 = var11 = var10000 ^ var15 + var16[var10003];
         var15 = src1 << 1 | src1 >>> 31;
         var10002 = var7 + var13;
         var7 = srcIndex + -1;
         src1 = var15 ^ var10002 + var14[srcIndex];
         srcIndex = var10000 >>> 1 | var11 << 31;
      }

      this.Bits32ToBytes(var5 ^ this.gSubKeys[0], dst, dstIndex);
      this.Bits32ToBytes(var6 ^ this.gSubKeys[1], dst, dstIndex + 4);
      this.Bits32ToBytes(src1 ^ this.gSubKeys[2], dst, dstIndex + 8);
      this.Bits32ToBytes(srcIndex ^ this.gSubKeys[3], dst, dstIndex + 12);
   }

   private final int F32(int x, int[] k32) {
      int[] var10001 = k32;
      int[] var10002 = k32;
      int[] var10003 = k32;
      int[] var10004 = k32;
      int var10006 = x;
      int var10008 = x;
      int var10010 = x;
      x = this.b0(x);
      int k321 = this.b1(var10010);
      int var3 = this.b2(var10008);
      int var4 = this.b3(var10006);
      int var5 = var10004[0];
      int var6 = var10003[1];
      int var7 = var10002[2];
      int var8 = var10001[3];
      int var9 = 0;
      byte[][] var10000;
      switch (this.k64Cnt & 3) {
         case 0:
            var10000 = P;
            x = var10000[1][x] & 255 ^ this.b0(var8);
            k321 = var10000[0][k321] & 255 ^ this.b1(var8);
            var3 = var10000[0][var3] & 255 ^ this.b2(var8);
            var4 = var10000[1][var4] & 255 ^ this.b3(var8);
         case 3:
            var10000 = P;
            x = var10000[1][x] & 255 ^ this.b0(var7);
            k321 = var10000[1][k321] & 255 ^ this.b1(var7);
            var3 = var10000[0][var3] & 255 ^ this.b2(var7);
            var4 = var10000[0][var4] & 255 ^ this.b3(var7);
         case 2:
            byte[][] var12;
            var9 = this.gMDS0[(var12 = P)[0][var12[0][x] & 255 ^ this.b0(var6)] & 255 ^ this.b0(var5)] ^ this.gMDS1[var12[0][var12[1][k321] & 255 ^ this.b1(var6)] & 255 ^ this.b1(var5)] ^ this.gMDS2[var12[1][var12[0][var3] & 255 ^ this.b2(var6)] & 255 ^ this.b2(var5)] ^ this.gMDS3[var12[1][var12[1][var4] & 255 ^ this.b3(var6)] & 255 ^ this.b3(var5)];
            break;
         case 1:
            byte[][] var11;
            var9 = this.gMDS0[(var11 = P)[0][x] & 255 ^ this.b0(var5)] ^ this.gMDS1[var11[0][k321] & 255 ^ this.b1(var5)] ^ this.gMDS2[var11[1][var3] & 255 ^ this.b2(var5)] ^ this.gMDS3[var11[1][var4] & 255 ^ this.b3(var5)];
      }

      return var9;
   }

   private final int RS_MDS_Encode(int k0, int k1) {
      for(int var3 = 0; var3 < 4; ++var3) {
         k1 = this.RS_rem(k1);
      }

      k0 ^= k1;

      for(k1 = 0; k1 < 4; ++k1) {
         k0 = this.RS_rem(k0);
      }

      return k0;
   }

   private final int RS_rem(int x) {
      int var5;
      int var10000 = var5 = x >>> 24 & 255;
      int var2 = var10000 << 1;
      int var3;
      if ((var10000 & 128) != 0) {
         var3 = 333;
      } else {
         var3 = 0;
      }

      var2 = (var2 ^ var3) & 255;
      var3 = var5 >>> 1;
      short var4;
      if ((var5 & 1) != 0) {
         var4 = 166;
      } else {
         var4 = 0;
      }

      var10000 = x;
      x = var3 ^ var4 ^ var2;
      return var10000 << 8 ^ x << 24 ^ var2 << 16 ^ x << 8 ^ var5;
   }

   private final int LFSR1(int x) {
      int var2 = x >> 1;
      short x1;
      if ((x & 1) != 0) {
         x1 = 180;
      } else {
         x1 = 0;
      }

      return var2 ^ x1;
   }

   private final int LFSR2(int x) {
      int var3 = x >> 2;
      short var2;
      if ((x & 2) != 0) {
         var2 = 180;
      } else {
         var2 = 0;
      }

      var3 ^= var2;
      byte x1;
      if ((x & 1) != 0) {
         x1 = 90;
      } else {
         x1 = 0;
      }

      return var3 ^ x1;
   }

   private final int Mx_X(int x) {
      return x ^ this.LFSR2(x);
   }

   private final int Mx_Y(int x) {
      return x ^ this.LFSR1(x) ^ this.LFSR2(x);
   }

   private final int b0(int x) {
      return x & 255;
   }

   private final int b1(int x) {
      return x >>> 8 & 255;
   }

   private final int b2(int x) {
      return x >>> 16 & 255;
   }

   private final int b3(int x) {
      return x >>> 24 & 255;
   }

   private final int Fe32_0(int x) {
      int[] var2;
      return (var2 = this.gSBox)[(x & 255) * 2 + 0] ^ var2[(x >>> 8 & 255) * 2 + 1] ^ var2[(x >>> 16 & 255) * 2 + 512] ^ var2[(x >>> 24 & 255) * 2 + 513];
   }

   private final int Fe32_3(int x) {
      int[] var2;
      return (var2 = this.gSBox)[(x >>> 24 & 255) * 2 + 0] ^ var2[(x & 255) * 2 + 1] ^ var2[(x >>> 8 & 255) * 2 + 512] ^ var2[(x >>> 16 & 255) * 2 + 513];
   }

   private final int BytesTo32Bits(byte[] b, int p) {
      return b[p] & 255 | (b[p + 1] & 255) << 8 | (b[p + 2] & 255) << 16 | (b[p + 3] & 255) << 24;
   }

   private final void Bits32ToBytes(int in, byte[] b, int offset) {
      b[offset] = (byte)in;
      int var4 = offset + 1;
      b[var4] = (byte)(in >> 8);
      var4 = offset + 2;
      b[var4] = (byte)(in >> 16);
      var4 = offset + 3;
      b[var4] = (byte)(in >> 24);
   }

   public void init(boolean encrypting, CipherParameters params) {
      if (params instanceof KeyParameter) {
         TwofishEngine var10000 = this;
         TwofishEngine var10001 = this;
         this.encrypting = encrypting;
         byte[] var3;
         this.workingKey = var3 = ((KeyParameter)params).getKey();
         var10001.k64Cnt = var3.length / 8;
         var10000.setKey(var3);
      } else {
         throw new IllegalArgumentException("invalid parameter passed to Twofish init - " + params.getClass().getName());
      }
   }

   public String getAlgorithmName() {
      return "Twofish";
   }

   public final int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this.workingKey != null) {
         if (inOff + 16 <= in.length) {
            if (outOff + 16 <= out.length) {
               if (this.encrypting) {
                  this.encryptBlock(in, inOff, out, outOff);
               } else {
                  this.decryptBlock(in, inOff, out, outOff);
               }

               return 16;
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException("Twofish not initialised");
      }
   }

   public void reset() {
      byte[] var1;
      if ((var1 = this.workingKey) != null) {
         this.setKey(var1);
      }

   }

   public int getBlockSize() {
      return 16;
   }
}
